@@ -9,11 +9,13 @@ import json
 import weakref
 
 from abc import abstractmethod
+from contextlib import contextmanager, suppress
 from contextvars import ContextVar
 from typing import (
     Any,
     Callable,
     Dict,
+    Generator,
     Iterable,
     List,
     Optional,
@@ -25,11 +27,16 @@ from typing import (
 )
 from weakref import ReferenceType
 
+# Third Party Library
+from typing_extensions import Literal
+
 
 var_root: ContextVar[Any] = ContextVar("root")
+var_parent: ContextVar[Union[List[Any], Dict[str, Any]]] = ContextVar("parent")
 T_SELF_VALUE = Union[Tuple[int, Any], Tuple[str, Any]]
 var_self: ContextVar[T_SELF_VALUE] = ContextVar("self")
 var_finding: ContextVar[bool] = ContextVar("finding", default=False)
+T_VALUE = Union[int, float, str, Literal[None], Literal[True], Literal[False]]
 
 T = TypeVar("T", bound="Expr")
 
@@ -71,23 +78,52 @@ class JSONPathFindError(JSONPathError):
     """
 
 
-def _dfs_find(
-    expr: Optional["Expr"], elements: List[Any], rv: List[Any]
-) -> None:
+@contextmanager
+def temporary_set(
+    context_var: ContextVar[Any], value: Any
+) -> Generator[None, None, None]:
+    """
+    Set the context variable temporarily via the 'with' statement.
+
+    >>> var_boo = ContextVar("boo")
+    >>> with temporary_set(var_boo, True):
+    ...     assert var_boo.get() is True
+    >>> var_boo.get()
+    Traceback (most recent call last):
+        ...
+    LookupError: ...
+    """
+    token = context_var.set(value)
+    try:
+        yield
+    finally:
+        context_var.reset(token)
+
+
+def _dfs_find(expr: "Expr", elements: List[Any], rv: List[Any]) -> None:
     """
     use DFS to find all target elements.
     the next expr finds in the result found by the current expr.
     """
-    if expr is None:
-        rv.extend(elements)
-    else:
-        for element in elements:
-            try:
-                _dfs_find(
-                    expr.get_next(), expr.find(element), rv,
-                )
-            except JSONPathFindError:
-                pass
+    next_expr = expr.get_next()
+    for element in elements:
+        try:
+            found_elements = expr.find(element)
+        except JSONPathFindError:
+            continue
+
+        if not found_elements:
+            continue
+
+        if next_expr is None:
+            # collect all found elements if there is no next expr.
+            rv.extend(found_elements)
+            continue
+
+        with temporary_set(var_parent, element):
+            _dfs_find(
+                next_expr, found_elements, rv,
+            )
 
 
 class ExprMeta(type):
@@ -132,12 +168,11 @@ class ExprMeta(type):
                 # but only the first time finding can set the root element.
                 token_root = var_root.set(element)
 
-            token_finding = var_finding.set(True)
             try:
-                _dfs_find(begin, [element], rv)
+                with temporary_set(var_finding, True):
+                    _dfs_find(begin, [element], rv)
                 return rv
             finally:
-                var_finding.reset(token_finding)
                 if token_root:
                     var_root.reset(token_root)
 
@@ -307,6 +342,43 @@ class Expr(metaclass=ExprMeta):
         return self.NotEqual(value)
 
 
+class Value(Expr):
+    """
+    Represent the value in the expression.
+
+    It is used mainly to support parsing comparison expression
+    which value is on the left.
+
+    >>> p = Value("boo"); print(p)
+    "boo"
+    >>> p.find([])
+    ['boo']
+    >>> print(Value(True))
+    true
+    >>> print(Value(False))
+    false
+    >>> print(Value(None))
+    null
+    >>> print(Value(1))
+    1
+    >>> print(Value(1.1))
+    1.1
+    >>> print(Value(1).LessThan(Value(2)))
+    1 < 2
+
+    """
+
+    def __init__(self, value: T_VALUE) -> None:
+        super().__init__()
+        self.value = value
+
+    def _get_partial_expression(self) -> str:
+        return json.dumps(self.value)
+
+    def find(self, element: Any) -> List[Any]:
+        return [self.value]
+
+
 class Root(Expr):
     """
     Represent the root of data.
@@ -414,16 +486,16 @@ class Array(Expr):
             return f"[{self.idx!s}]"
 
     def find(self, element: Any) -> List[Any]:
-        if self.idx is None and isinstance(element, list):
-            return element
-        elif (
-            isinstance(self.idx, int)
-            and isinstance(element, list)
-            and self.idx < len(element)
-        ):
-            return [element[self.idx]]
-        elif isinstance(self.idx, Slice):
-            return self.idx.find(element)
+        if isinstance(element, list):
+            if self.idx is None:
+                return element
+            elif isinstance(self.idx, int):
+                with suppress(IndexError):
+                    return [element[self.idx]]
+            elif isinstance(self.idx, Slice):
+                return self.idx.find(element)
+            else:
+                raise AssertionError(f"self.idx={self.idx!r} is not valid")
 
         raise JSONPathFindError
 
@@ -476,18 +548,15 @@ class Predicate(Expr):
 
         for item in items:
             # save the current item into var_self for Self()
-            token_self = var_self.set(item)
             # set var_finding False to
             # start new finding process for the nested expr: self.idx
-            token_finding = var_finding.set(False)
-            _, value = item
-            try:
+            with temporary_set(var_self, item), temporary_set(
+                var_finding, False
+            ):
+                _, value = item
                 rv = self.expr.find(value)
                 if rv and rv[0]:
                     filtered_items.append(value)
-            finally:
-                var_finding.reset(token_finding)
-                var_self.reset(token_self)
         return filtered_items
 
 
@@ -501,9 +570,9 @@ class Slice(Expr):
 
     def __init__(
         self,
-        start: Optional[int] = None,
-        stop: Optional[int] = None,
-        step: Optional[int] = None,
+        start: Union[Expr, int, None] = None,
+        stop: Union[Expr, int, None] = None,
+        step: Union[Expr, int, None] = None,
     ) -> None:
         super().__init__()
         self.start = start
@@ -527,21 +596,34 @@ class Slice(Expr):
 
         return ":".join(parts)
 
-    def find(self, element: List[Any]) -> Any:
-        if isinstance(element, list):
-            start = self.start
-            end = self.end
-            step = self.step
-            if start is None:
-                start = 0
-            if end is None:
-                end = len(element)
-            if step is None:
-                step = 1
+    def _ensure_int_or_none(
+        self, value: Union[Expr, int, None]
+    ) -> Union[int, None]:
+        if isinstance(value, Expr):
+            # set var_finding False to start new finding process for the nested expr
+            with temporary_set(var_finding, False):
+                found_elements = value.find(var_parent.get())
+            if not found_elements or not isinstance(found_elements[0], int):
+                raise JSONPathFindError
+            return found_elements[0]
+        else:
+            return value
 
-            return element[start:end:step]
+    def find(self, element: List[Any]) -> List[Any]:
+        assert isinstance(element, list), "Slice.find apply on list only."
 
-        raise JSONPathFindError
+        start = self._ensure_int_or_none(self.start)
+        end = self._ensure_int_or_none(self.end)
+        step = self._ensure_int_or_none(self.step)
+
+        if start is None:
+            start = 0
+        if end is None:
+            end = len(element)
+        if step is None:
+            step = 1
+
+        return element[start:end:step]
 
 
 class Brace(Expr):
@@ -571,11 +653,8 @@ class Brace(Expr):
     def find(self, element: Any) -> List[Any]:
         # set var_finding False to
         # start new finding process for the nested expr: self.expr
-        token = var_finding.set(False)
-        try:
+        with temporary_set(var_finding, False):
             return [self._expr.find(element)]
-        finally:
-            var_finding.reset(token)
 
 
 def _recursive_find(expr: Expr, element: Any, rv: List[Any]) -> None:
@@ -587,12 +666,14 @@ def _recursive_find(expr: Expr, element: Any, rv: List[Any]) -> None:
         rv.extend(find_rv)
     except JSONPathFindError:
         pass
-    if isinstance(element, list):
-        for item in element:
-            _recursive_find(expr, item, rv)
-    elif isinstance(element, dict):
-        for item in element.values():
-            _recursive_find(expr, item, rv)
+
+    with temporary_set(var_parent, element):
+        if isinstance(element, list):
+            for item in element:
+                _recursive_find(expr, item, rv)
+        elif isinstance(element, dict):
+            for item in element.values():
+                _recursive_find(expr, item, rv)
 
 
 class Search(Expr):
@@ -688,10 +769,9 @@ class Compare(Expr):
 
     def get_target_value(self) -> Any:
         if isinstance(self.target, Expr):
-            try:
-                # set var_finding False to
-                # start new finding process for the nested expr: self.target
-                token = var_finding.set(False)
+            # set var_finding False to
+            # start new finding process for the nested expr: self.target
+            with temporary_set(var_finding, False):
                 # multiple exprs begins on self-value in filtering find,
                 # except the self.target expr starts with root-value.
                 _, value = var_self.get()
@@ -700,9 +780,6 @@ class Compare(Expr):
                     raise JSONPathFindError
 
                 return rv[0]
-            finally:
-                var_finding.reset(token)
-
         else:
             return self.target
 
@@ -883,11 +960,8 @@ class Contains(Function):
         if isinstance(target_arg, Expr):
             # set var_finding False to
             # start new finding process for the nested expr: target_arg
-            token = var_finding.set(False)
-            try:
+            with temporary_set(var_finding, False):
                 rv = self._target.find(element)
-            finally:
-                var_finding.reset(token)
 
             if not rv:
                 return []
@@ -921,11 +995,8 @@ class Not(Function):
     def find(self, element: Any) -> List[bool]:
         # set var_finding False to
         # start new finding process for the nested expr: target
-        token = var_finding.set(False)
-        try:
+        with temporary_set(var_finding, False):
             rv = self._expr.find(element)
-        finally:
-            var_finding.reset(token)
 
         return [not v for v in rv]
 
@@ -953,4 +1024,5 @@ __all__ = (
     "Search",
     "Self",
     "Slice",
+    "Value",
 )
